//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#ifndef SENSOR_INERTIAL_KINEMATICS_H
#define SENSOR_INERTIAL_KINEMATICS_H

//wolf includes
#include "core/sensor/sensor_base.h"
#include "core/utils/params_server.h"
#include <iostream>

namespace wolf {

WOLF_STRUCT_PTR_TYPEDEFS(ParamsSensorInertialKinematics);


struct ParamsSensorInertialKinematics : public ParamsSensorBase
{
        //noise std dev
        double std_pbc;   // standard deviation of the com position measurement relative to the base frame (m)
        double std_vbc;   // standard deviation of the com velocity measurement relative to the base frame (m/s)

        ParamsSensorInertialKinematics() = default;
        ParamsSensorInertialKinematics(std::string _unique_name, const ParamsServer& _server):
            ParamsSensorBase(_unique_name, _server)
        {
            std_pbc          = _server.getParam<double>(_unique_name + "/std_pbc");
            std_vbc          = _server.getParam<double>(_unique_name + "/std_vbc");
        }
        ~ParamsSensorInertialKinematics() override = default;
        std::string print() const override
        {
            return ParamsSensorBase::print()                                           + "\n"
                    + "std_pbc: "           + std::to_string(std_pbc)           + "\n"
                    + "std_vbc: "           + std::to_string(std_vbc)           + "\n";
        }
};

WOLF_PTR_TYPEDEFS(SensorInertialKinematics);

class SensorInertialKinematics : public SensorBase
{

    protected:
        ParamsSensorInertialKinematicsPtr intr_ikin_;

    public:

        SensorInertialKinematics(const Eigen::VectorXd& _extrinsics, ParamsSensorInertialKinematicsPtr _intr_ikin);
        ~SensorInertialKinematics() override;

        WOLF_SENSOR_CREATE(SensorInertialKinematics, ParamsSensorInertialKinematics, 0);

        double getPbcNoiseStd() const;
        double getVbcNoiseStd() const;
};

inline double SensorInertialKinematics::getPbcNoiseStd() const
{
    return intr_ikin_->std_pbc;
}

inline double SensorInertialKinematics::getVbcNoiseStd() const
{
    return intr_ikin_->std_vbc;
}

} // namespace wolf

#endif // SENSOR_INERTIAL_KINEMATICS_H
